use std::rc::Rc;
use futures::executor::block_on;
use gio::{Menu, MenuItem, SimpleAction, SimpleActionGroup};
use gtk4::builders::PopoverMenuBuilder;
use gtk4::{prelude::*, PopoverMenu, AboutDialog, License};
use gtk4::subclass::prelude::*;
use gtk4::{
    gdk::Paintable,
    glib::*,
    ApplicationWindow,
    HeaderBar,
    Button,
    Window,
    Popover,
    Builder,
    IconTheme,
    IconLookupFlags,
    TextDirection,
    MenuButton,
};

use crate::popover::CustomPopoverMenu;

#[derive(Debug)]
pub struct CustomTitleBar {
    pub window: Window,
    pub titlebar: HeaderBar,
    pub run_button: Button,
    pub menu_button: MenuButton,
    pub menu: Menu,
    pub menu_actions: SimpleActionGroup,
}

impl CustomTitleBar {
    pub fn new() -> CustomTitleBar {
        // loading PopoverMenu logic goes here
        CustomTitleBar {
            window: Window::new(),
            titlebar: HeaderBar::new(),
            run_button: Button::new(),
            menu_button: MenuButton::new(),
            menu: Menu::new(),
            menu_actions: SimpleActionGroup::new(),
        }
    }

    pub fn set_buttons(&self) {
        // set labels
        self.run_button.set_label("Apply Patch");
        //self.menu_button.set_label("Menu");

        // set css class; it turns blue!
        self.run_button.set_css_classes(&["run_btn"]);

        // set visibility
        self.run_button.set_visible(true);
        self.menu_button.set_visible(true);

        // symbolic
        // let menu_button = IconTheme::new();
        // menu_button.lookup_icon("open-menu-symbolic", &[], 16, 1, TextDirection::Ltr, IconLookupFlags::empty());
        //self.menu_button.set_popover(Some(&self.popover_menu.popover));
        self.menu.append(Some("About"), Some("about_action"));
        self.menu.append(Some("Keyboard Shortcuts"), None);
        self.menu.append(Some("Quit"), None);
        let popover_menu = Some(PopoverMenu::builder().menu_model(&self.menu).build());
        self.menu_button.set_popover(Some(&popover_menu.unwrap()));
    }

    pub fn set_actions(&self) {
        // about action
        let about_action = SimpleAction::new("about_action", None);
        about_action.set_enabled(true);
        about_action.connect_activate(clone!(@strong self as this =>
            move |_, _| {
                let about_dialog = AboutDialog::builder()
                    .program_name("SNESPatcher")
                    .version("0.1.0")
                    .comments("GTK4 app to patch your SNES ROMs!")
                    .authors(["Wyatt J. Miller".to_string()].to_vec())
                    .documenters(["Wyatt J. Miller".to_string()].to_vec())
                    .license_type(License::Mpl20)
                    .website("https://scm.wyattjmiller.com/wymiller/snespatcher")
                    .website_label("Website?")
                    .modal(false)
                    .can_focus(true)
                    //.child(&window)
                    .build();
                about_dialog.show();
        }));

        // add actions to the menu
        self.menu_actions.add_action(&about_action);
        //println!("{:?}", self.menu_actions.is_action_enabled("About"));
    }

    pub fn click_run_button(&self) {
        self.run_button.connect_clicked(move |button| {
            button.set_label("Avery is awesome!");
        });
    }

    pub fn set_window(&self, window: &ApplicationWindow) {
        window.set_titlebar(Some(&self.titlebar));
    }

    pub fn set_titlebar(&self) {
        self.titlebar.pack_start(&self.run_button);
        self.titlebar.pack_end(&self.menu_button);
        self.titlebar.set_css_classes(&["headerbar"])
    }

    pub fn set_popover(&self) {
        // huh? why is this method here again?
    }
}